@@ -17,10 +17,10 @@
 //!    The result will be printed to standard output and the result returned so that `DEBUG` does not affect the computation.    
 #![allow(clippy::needless_return)]
 
-use sxd_document::dom::{Element, ChildOfElement, ParentOfChild};
+use sxd_document::dom::{Element, ChildOfElement};
 use sxd_xpath::{Value, Context, context, function::*, nodeset::*};
-use crate::{canonicalize::as_text, definitions::{ DEFINITIONS}};
-use regex::{Regex, Captures};
+use crate::definitions::DEFINITIONS;
+use regex::Regex;
 use crate::pretty_print::mml_to_string;
 use std::cell::Ref;
 
@@ -52,7 +52,7 @@ fn get_text_from_COE(coe: &ChildOfElement) -> String {
 
 // make sure that there is only one node in the NodeSet
 // Returns the node or an Error
-fn validate_one_node<'n>(nodes: Nodeset<'n>, func_name: &str) -> Result<Node<'n>, Error> {
+pub fn validate_one_node<'n>(nodes: Nodeset<'n>, func_name: &str) -> Result<Node<'n>, Error> {
     if nodes.size() == 0 {
         return Err(Error::Other(format!("Missing argument for {}", func_name)));
     } else if nodes.size() > 1 {
@@ -648,260 +648,6 @@ impl Function for Max {
 }
 
 
-struct NestingChars;
-const NEMETH_FRAC_LEVEL: &'static str = "nemeth-frac-level";    // name of attr where value is cached
-const FIRST_CHILD_ONLY: &[&str] = &["mroot", "msub", "msup", "msubsup", "munder", "mover", "munderover", "mmultiscripts"];
-impl NestingChars {
-    // returns a 'repeat_char' corresponding to the Nemeth rules for nesting
-    // note: this value is likely one char too long because the starting fraction is counted
-    fn nemeth_frac_value<'a>(node: &'a Element, repeat_char: &'a str) -> String {
-        let children = node.children();
-        let name = name(&node);
-        if MATHML_LEAF_NODES.contains(&name) {
-            return "".to_string();
-        } else if name == "mfrac" {
-            // have we already computed the value?
-            if let Some(value) = node.attribute_value(NEMETH_FRAC_LEVEL) {
-                return value.to_string();
-            }
-
-            let num_value = NestingChars::nemeth_frac_value(&as_element(children[0]), repeat_char);
-            let denom_value = NestingChars::nemeth_frac_value(&as_element(children[1]), repeat_char);
-            let mut max_value = if num_value.len() > denom_value.len() {num_value} else {denom_value};
-            max_value += repeat_char;
-            node.set_attribute_value(NEMETH_FRAC_LEVEL, &max_value);
-            return max_value;
-        } else if FIRST_CHILD_ONLY.contains(&name) {
-            // only look at the base -- ignore scripts/index
-            return NestingChars::nemeth_frac_value(&as_element(children[0]), repeat_char);
-        } else {
-            let mut result = "".to_string();
-            for child in children {
-                let value = NestingChars::nemeth_frac_value(&as_element(child), repeat_char);
-                if value.len() > result.len() {
-                    result = value;
-                }
-            }
-            return result;
-        }
-    }
-
-    fn nemeth_root_value<'a>(node: &'a Element, repeat_char: &'a str) -> Result<String, Error> {
-        // returns the correct number of repeat_chars to use
-        // note: because the highest count is toward the leaves and
-        //    because this is a loop and not recursive, caching doesn't work without a lot of overhead
-        let parent = node.parent().unwrap();
-        if let ParentOfChild::Element(e) =  parent {
-            let mut parent = e;
-            let mut result = "".to_string();
-            loop {
-                let name = name(&parent);
-                if name == "math" {
-                    return Ok( result );
-                }
-                if name == "msqrt" || name == "mroot" {
-                    result += repeat_char;
-                }
-                let parent_of_child = parent.parent().unwrap();
-                if let ParentOfChild::Element(e) =  parent_of_child {
-                    parent = e;
-                } else {
-                    return Err( Error::Other("Internal error in nemeth_root_value: didn't find 'math' tag".to_string()) );
-                }
-            }
-        }
-        return Err( Error::Other("Internal error in nemeth_root_value: didn't find 'math' tag".to_string()) );
-    }
-}
-
-impl Function for NestingChars {
-/**
- * Returns a string with the correct number of nesting chars (could be an empty string)
- * @param(node) -- current node
- * @param(char) -- char (string) that should be repeated
- * Note: as a side effect, an attribute with the value so repeated calls to this or a child will be fast
- */
- fn evaluate<'c, 'd>(&self,
-                        _context: &context::Evaluation<'c, 'd>,
-                        args: Vec<Value<'d>>)
-                        -> Result<Value<'d>, Error>
-    {
-        let mut args = Args(args);
-        args.exactly(2)?;
-        let repeat_char = args.pop_string()?;
-        let node = validate_one_node(args.pop_nodeset()?, "NestingChars")?;
-        if let Node::Element(el) = node {
-            let name = name(&el);
-            // it is likely a bug to call this one a non mfrac
-            if name == "mfrac" {
-                // because it is called on itself, the fraction is counted one too many times -- chop one off
-                // this is slightly messy because we are chopping off a char, not a byte
-                const BRAILLE_BYTE_LEN: usize = "⠹".len();      // all Unicode braille symbols have the same number of bytes
-                return Ok( Value::String( NestingChars::nemeth_frac_value(&el, &repeat_char)[BRAILLE_BYTE_LEN..].to_string() ) );
-            } else if name == "msqrt" || name == "mroot" {
-                return Ok( Value::String( NestingChars::nemeth_root_value(&el, &repeat_char)? ) );
-            } else {
-                panic!("NestingChars chars should be used only on 'mfrac'. '{}' was passed in", name);
-            }
-        } else {
-            // not an element, so nothing to do
-            return Ok( Value::String("".to_string()) );
-        }
-    }
-}
-
-struct NemethChars;
-impl NemethChars {
-    // returns a string for the chars in the *leaf* node.
-    // this string follows the Nemeth rules typefaces and deals with mathvariant
-    //  which has partially turned chars to the alphanumeric block
-    fn get_nemeth_chars<'a>(node: &'a Element) -> Result<String, Error> {
-        lazy_static! {
-            // To greatly simplify typeface/language generation, the chars have unique ASCII chars for them:
-            // Typeface: S: sans-serif, B: bold, T: script/blackboard, I: italic, R: Roman
-            // Language: E: English, D: German, G: Greek, V: Greek variants, H: Hebrew, U: Russian
-            // Indicators: C: capital, L: letter, N: number, P: punctuation, M: multipurpose
-            static ref PICK_APART_CHAR: Regex = 
-                Regex::new(r"(?P<face>[SBTIR]*)(?P<lang>[EDGVHU]?)(?P<cap>C?)(?P<letter>L?)(?P<num>[N]?)(?P<char>.)").unwrap();
-        }
-    
-        let math_variant = node.attribute_value("mathvariant");
-        // FIX: cover all the options -- use phf::Map
-        let  attr_typeface = match math_variant {
-            None => "R",
-            Some(variant) => match variant {
-                "bold" => "B",
-                "italic" => "I",
-                "double-struck" | "script" => "T",
-                "fraktur" => "G",
-                "sans-serif" => "S",
-                _ => "R",       // normal and unknown
-            },
-        };
-        let text = as_text(*node);
-        let braille_chars = crate::speech::braille_replace_chars(text, *node).unwrap_or("".to_string());
-        // debug!("braille_chars: '{}'", braille_chars);
-        
-        // we want to pull the prefix (typeface, language) out to the front until a change happens
-        // the same is true for number indicator
-        // also true (sort of) for capitalization -- if all caps, use double cap in front (assume abbr or Roman Numeral)
-        let is_in_enclosed_list = name(node) == "mn" && NemethChars::is_in_enclosed_list(*node);
-        let mut typeface = "R".to_string();     // assumption is "R" and if attr or letter is different, something happens
-        let mut is_all_caps = true;
-        let mut is_all_caps_valid = false;      // all_caps only valid if we did a replacement
-        let result = PICK_APART_CHAR.replace_all(&braille_chars, |caps: &Captures| {
-            // debug!("  face: {:?}, lang: {:?}, num {:?}, cap: {:?}, char: {:?}",
-            //        &caps["face"], &caps["lang"], &caps["num"], &caps["cap"], &caps["char"]);
-            let mut nemeth_chars = "".to_string();
-            let char_face = if caps["face"].is_empty() {attr_typeface} else {&caps["face"]};
-            let typeface_changed =  &typeface != char_face;
-            if typeface_changed {
-                typeface = char_face.to_string();   // needs to outlast this instance of the loop
-                nemeth_chars += &typeface;
-                nemeth_chars +=  &caps["lang"];
-            } else {
-                nemeth_chars +=  &caps["lang"];
-            }
-            // debug!("  typeface changed: {}, is_in_list: {}; num: {}", typeface_changed, is_in_enclosed_list, !caps["num"].is_empty());
-            if !caps["num"].is_empty() && (typeface_changed || !is_in_enclosed_list) {
-                nemeth_chars += "N";
-            }
-            is_all_caps_valid = true;
-            is_all_caps &= !&caps["cap"].is_empty();
-            nemeth_chars += &caps["cap"];       // will be stripped later if all caps
-            nemeth_chars += &caps["letter"];
-            nemeth_chars += &caps["char"];
-            return nemeth_chars;
-        });
-        // debug!("  result: {}", &result);
-        let mut text_chars = text.chars();     // see if more than one char
-        if is_all_caps_valid && is_all_caps && text_chars.next().is_some() &&  text_chars.next().is_some() {
-            return Ok( "CC".to_string() + &result.replace("C", ""));
-        } else {
-            return Ok( result.to_string() );
-        }
-    }
-
-    fn is_in_enclosed_list(node: Element) -> bool {
-        // Nemeth Rule 10 defines an enclosed list:
-        // 1: begins and ends with fence
-        // 2: FIX: not implemented -- must contain no word, abbreviation, ordinal or plural ending
-        // 3: function names or signs of shape and the signs which follow them are a single item (not a word)
-        // 4: an item of the list may be an ellipsis or any sign used for mission
-        // 5: no relational operator may appear within the list
-        // 6: the list must have at least 2 items.
-        //       Items are separated by commas, can not have other punctuation (except ellipsis and dash)
-        let mut parent = node.parent().unwrap().element().unwrap(); // safe since 'math' is always at root
-        while name(&parent) == "mrow" {
-            if IsBracketed::is_bracketed(&parent, "", "", true, false) {
-                for child in parent.children() {
-                    if !child_meets_conditions(as_element(child)) {
-                        return false;
-                    }
-                }
-                return true;
-            }
-            parent = parent.parent().unwrap().element().unwrap();
-        }
-        return false;
-
-        fn child_meets_conditions(node: Element) -> bool {
-            let name = name(&node);
-            return match name {
-                "mi" | "mn" => true,
-                "mo"  => !crate::canonicalize::is_relational_op(node),
-                "mtext" => false, // FIX -- should be more nuanced,
-                "mrow" => {
-                    if IsBracketed::is_bracketed(&node, "", "", false, false) {
-                        return child_meets_conditions(as_element(node.children()[1]));
-                    } else {
-                        for child in node.children() {
-                            if !child_meets_conditions(as_element(child)) {
-                                return false;
-                            }
-                        }
-                    }  
-                    true      
-                },
-                _ => {
-                    for child in node.children() {
-                        if !child_meets_conditions(as_element(child)) {
-                            return false;
-                        }
-                    }
-                    true
-                },
-            }
-        }
-    }
-}
-
-impl Function for NemethChars {
-    /**
-     * Returns a string with the correct number of nesting chars (could be an empty string)
-     * @param(node) -- current node
-     * @param(char) -- char (string) that should be repeated
-     * Note: as a side effect, an attribute with the value so repeated calls to this or a child will be fast
-     */
-     fn evaluate<'c, 'd>(&self,
-                            _context: &context::Evaluation<'c, 'd>,
-                            args: Vec<Value<'d>>)
-                            -> Result<Value<'d>, Error>
-        {
-            let mut args = Args(args);
-            args.exactly(1)?;
-            let node = validate_one_node(args.pop_nodeset()?, "NemethChars")?;
-            if let Node::Element(el) = node {
-                assert!( MATHML_LEAF_NODES.contains(&name(&el)) );
-                return Ok( Value::String( NemethChars::get_nemeth_chars(&el)? ) );
-            } else {
-                // not an element, so nothing to do
-                return Ok( Value::String("".to_string()) );
-            }
-        }
-    }
-    
-
 struct IsLargeOp;
 /**
  * Returns true if the node is a large op
@@ -1284,8 +1030,8 @@ pub fn add_builtin_functions(context: &mut Context) {
     // FIX: should be a static cache that gets regenerated on update
     context.set_function("min", Min);       // missing in xpath 1.0
     context.set_function("max", Max);       // missing in xpath 1.0
-    context.set_function("NestingChars", NestingChars);
-    context.set_function("NemethChars", NemethChars);
+    context.set_function("NestingChars", crate::braille::NemethNestingChars);
+    context.set_function("BrailleChars", crate::braille::BrailleChars);
     context.set_function("IsNode", IsNode);
     context.set_function("ToOrdinal", ToOrdinal);
     context.set_function("ToCommonFraction", ToCommonFraction);
